@@ -9,15 +9,18 @@ from bnlearn.bnlearn import (
     compare_networks,
     plot,
     adjmat2vec,
+    adjmat2dict,
     vec2adjmat,
+    dag2adjmat,
     df2onehot,
     topological_sort,
     predict,
     query2df,
+    vec2df,
     get_node_properties,
     get_edge_properties,
-    _dag2adjmat,
     _filter_df,
+    independence_test,
     save,
     load,
 )
@@ -32,7 +35,7 @@ from packaging import version
 
 __author__ = 'Erdogan Tasksen'
 __email__ = 'erdogant@gmail.com'
-__version__ = '0.4.11'
+__version__ = '0.6.1'
 
 try:
     import pgmpy
@@ -61,17 +64,13 @@ BNLEARN - bnlearn is an Python package for learning the graphical structure of B
 Description
 -----------
 * Learning a Bayesian network can be split into two problems:
-    * Parameter learning: Given a set of data samples and a DAG that captures the dependencies between the variables,
-      estimate the (conditional) probability distributions of the individual variables.
     * Structure learning: Given a set of data samples, estimate a DAG that captures the dependencies between the variables.
-* Currently, the library supports:
-    * Parameter learning for *discrete* nodes:
-    * Maximum Likelihood Estimation
-    * Bayesian Estimation
-* Structure learning for *discrete*, *fully observed* networks:
-    * Score-based structure estimation (BIC/BDeu/K2 score; exhaustive search, hill climb/tabu search)
-    * Constraint-based structure estimation (PC)
-    * Hybrid structure estimation (MMHC)
+    * Parameter learning: Given a set of data samples and a DAG that captures the dependencies between the variables.
+    * Making inferences.
+    * Parameter and structure learning is for *discrete* nodes
+        * Score-based structure estimation (BIC/BDeu/K2 score; exhaustive search, hill climb/tabu search)
+        * Constraint-based structure estimation (PC)
+        * Hybrid structure estimation (MMHC)
 
 
 Example
@@ -79,10 +78,12 @@ Example
 >>> # Import library
 >>> import bnlearn as bn
 >>> model = bn.import_DAG('sprinkler')
+>>> # Print CPDs
+>>> bn.print_CPD(model)
+>>> # Plot DAG
 >>> bn.plot(model)
 >>>
->>> # Import example
->>> df = bn.import_example('sprinkler')
+>>> # Sampling using DAG and CPDs
 >>> df = bn.sampling(model)
 >>>
 >>> # Do the inference
@@ -91,8 +92,18 @@ Example
 >>>
 >>> # Structure learning
 >>> model_sl = bn.structure_learning.fit(df)
+>>> # Compute edge strength using chi-square independence test
+>>> model_sl = bn.independence_test(model_sl, df)
+>>> # Plot DAG
+>>> bn.plot(model_sl)
+>>>
 >>> # Parameter learning
 >>> model_pl = bn.parameter_learning.fit(model_sl, df)
+>>> # Compute edge strength using chi-square independence test
+>>> model_pl = bn.independence_test(model_pl, df)
+>>> # Plot DAG
+>>> bn.plot(model_pl)
+>>>
 >>> # Compare networks
 >>> scores, adjmat = bn.compare_networks(model_sl, model)
 
@@ -100,5 +111,7 @@ References
 ----------
 * https://bnlearn.readthedocs.io
 * https://github.com/erdogant/bnlearn
+* https://towardsdatascience.com/a-step-by-step-guide-in-detecting-causal-relationships-using-bayesian-structure-learning-in-python-c20c6b31cee5
+* https://towardsdatascience.com/a-step-by-step-guide-in-designing-knowledge-driven-models-using-bayesian-theorem-7433f6fd64be
 
 """
