@@ -1,30 +1,291 @@
+import pandas as pd
+import numpy as np
+from pgmpy.inference import VariableElimination
+from pgmpy.models import BayesianNetwork, NaiveBayes
+from pgmpy.estimators import ExhaustiveSearch, HillClimbSearch, TreeSearch
+from pgmpy.factors.discrete import TabularCPD
+
 # %%
 import bnlearn as bn
 # print(bn.__version__)
-# print(dir(bn))
-
 # print(dir(bn.structure_learning))
 # print(dir(bn.parameter_learning))
 # print(dir(bn.inference))
 
+# %%
+
+# Load example dataset
+df = bn.import_example('sprinkler')
+
+edges = [('Cloudy', 'Sprinkler'),
+         ('Cloudy', 'Rain'),
+         ('Sprinkler', 'Wet_Grass'),
+         ('Rain', 'Wet_Grass')]
+
+# Make the actual Bayesian DAG
+DAG = bn.make_DAG(edges, verbose=0, methodtype='bayes')
+model = bn.parameter_learning.fit(DAG, df, verbose=3)
+bn.print_CPD(DAG)
+
+model = bn.parameter_learning.fit(DAG, df, verbose=3)
+bn.print_CPD(model)
+
+
+
+# %%
+import bnlearn as bn
+
+# Example dataset
+source=['Cloudy','Cloudy','Sprinkler','Rain']
+target=['Sprinkler','Rain','Wet_Grass','Wet_Grass']
+weights=[1,2,1,3]
+
+# Convert into sparse datamatrix
+df = bn.vec2df(source, target, weights=weights)
+# Make DAG
+DAG = bn.make_DAG(list(zip(source, target)), verbose=0)
+# Make plot
+bn.plot(DAG, interactive=True)
+
+
+# %%
+import bnlearn as bn
+
+raw = bn.import_example('stormofswords')
+# Convert raw data into sparse datamatrix
+df = bn.vec2df(raw['source'], raw['target'], raw['weight'])
+# Make the actual Bayesian DAG
+DAG = bn.make_DAG(list(zip(raw['source'], raw['target'])), verbose=0)
+# Make plot
+bn.plot(DAG, interactive=True)
+# Parameter learning
+model = bn.parameter_learning.fit(DAG, df, verbose=3)
+# Structure learning
+DAG_learned = bn.structure_learning.fit(df.iloc[:, 0:50])
+# Keep only significant edges
+DAG_learned = bn.independence_test(DAG_learned, df, prune=True)
+# Plot
+bn.plot(DAG_learned, interactive=True)
+
+# Generate some data based on DAG
+# df1 = bn.sampling(model, n=1000)
+# Make predictions
+# print(query)
+query = bn.inference.fit(DAG, variables=['Grenn'], evidence={'Aemon': 1, 'Samwell': 1})
+print(query)
+query.df
+
+# %%
+import bnlearn as bn
+df = bn.import_example()
+# Structure learning
+model = bn.structure_learning.fit(df, methodtype='tan', root_node='Cloudy', class_node='Rain', verbose=0)
+bn.plot(model)
+bn.plot(model, interactive=False, node_size=10)
+
+
+# %% Check the stochastic component of bnlearn
+DAG = bn.import_DAG('sprinkler', verbose=0)
+df = bn.import_example('sprinkler')
+adjmats = []
+
+for i in range(0, 10):
+    # print(i)
+    df = bn.sampling(DAG, n=10000)
+    model = bn.structure_learning.fit(df)
+    # df = bn.import_example()
+    # model = bn.structure_learning.fit(df)
+    adjmats.append(model['adjmat'].values.ravel())
+
+adjmats = np.array(adjmats)
+adjmat = (adjmats[:, None, :] != adjmats).sum(2)
+
+print(adjmat.sum(axis=0))
+# print(adjmat.sum(axis=1))
+
+
+# %% LOAD BIF FILE
+
+# Example 1: Plot the TRUE DAG
+DAG_1 = bn.import_DAG('sprinkler', verbose=0)
+graph = bn.plot(DAG_1)
+
+# Example 2: Download small spinkler example with 1000 samples
+df = bn.import_example('sprinkler')
+DAG_2 = bn.structure_learning.fit(df)
+bn.plot(DAG_2, pos=graph['pos'])
+
+# Example 3: Generate Download small spinkler example with 5000 samples
+df = bn.sampling(DAG_1, n=5000)
+DAG_3 = bn.structure_learning.fit(df)
+bn.plot(DAG_3, pos=graph['pos'])
+
+
+# %% LOAD BIF FILE
+DAG = bn.import_DAG('water', verbose=0)
+# Sampling
+df = bn.sampling(DAG, n=1000)
+# Parameter learning
+model = bn.parameter_learning.fit(DAG, df)
+print(len(model['model_edges']))
+# G = bn.plot(model)
+
+# Test for independence
+model1 = bn.independence_test(model, df, prune=False)
+# bn.plot(model1, pos=G['pos']);
+print(len(model1['model_edges']))
+print(model1['independence_test'].shape)
+
+# Test for independence
+model2 = bn.independence_test(model, df, prune=True)
+# bn.plot(model2, pos=G['pos']);
+print(len(model2['model_edges']))
+print(model2['independence_test'].shape)
+
+
+assert model['model_edges']==model1['model_edges']
+assert len(model1['model_edges'])==model1['independence_test'].shape[0]
+assert len(model2['model_edges'])==model2['independence_test'].shape[0]
+assert len(model2['model_edges'])<len(model1['model_edges'])
+assert len(model2['model_edges'])<len(model['model_edges'])
+
+# Plot
+# G = bn.plot(model1, interactive=True)
+# G = bn.plot(model2, interactive=True)
+
+
+# %% Adjust some edge properties
+# Load asia DAG
+df = bn.import_example(data='sprinkler')
+# Structure learning of sampled dataset
+model = bn.structure_learning.fit(df)
+# Compute edge strength with the chi_square test statistic
+model = bn.independence_test(model, df, test='chi_square', prune=True)
+# Make plot
+bn.plot(model)
+
+# %% Adjust some edge properties
+# Load asia DAG
+df = bn.import_example(data='asia')
+# Structure learning of sampled dataset
+model = bn.structure_learning.fit(df)
+# Make plot
+G = bn.plot(model)
+
+# Compute associations with the chi_square test statistic
+model1 = bn.independence_test(model, df, test='chi_square', prune=False)
+# Make plot
+bn.plot(model1, pos=G['pos'])
+
+# Compute associations with the chi_square test statistic
+model2 = bn.independence_test(model, df, test='chi_square', prune=True)
+# Make plot
+bn.plot(model2, pos=G['pos'])
+
+
+# %%
+# edges=list(model['model_edges'])
+
+# hn = hnet(multtest='holm', white_list=np.unique(edges))
+# hn.association_learning(df)
+
+# adjmat = hn.results['simmatP_cat'].copy()
+# adjmat = hn.results['simmatLogP_cat'].copy()
+# adjmat = adjmat.iloc[:, ismember(model['adjmat'].columns.values, adjmat.columns.values)[1]]
+# adjmat = adjmat.iloc[ismember(model['adjmat'].index.values, adjmat.index.values)[1], :]
+# adjmat = adjmat *model['adjmat']
+
+# assert np.all(model['adjmat'].index.values==adjmat.index.values)
+# assert np.all(model['adjmat'].columns.values==adjmat.columns.values)
+
+# model['adjmat'] = adjmat
+# # edge_properties = bn.get_edge_properties(model)
+# G = bn.plot(model)
+
+
+# scores = correlation_score(model['model'], df, test="chi_square", significance_level=0.05, return_summary=True)
+# f1score = correlation_score(model['model'], df, test="log_likelihood", significance_level=0.05, return_summary=False)
+# structure_score(model['model'], df, scoring_method='bic')
+
+
+# edges=list(model['model_edges'])
+
+
+# for edge in edges:
+#     P=[]
+#     print(edge)
+#     uiyc = df[edge[1]].unique()
+#     uiyc = uiyc[uiyc>0]
+
+#     uicats = df[edge[0]].unique()
+#     uicats = uicats[uicats>0]
+#     for uicat in uicats:
+#         for y in uiyc:
+#             outtest = _prob_hypergeo(df[edge[0]]==uicat, df[edge[1]]==y)
+#             print(outtest)
+#             P.append(outtest['P'])
+
+
+# y = df['Survived'].values
+# out = hn.enrichment(df, y)
+
+
+# # plot static
+# G = bn.plot(model)
+
+
+# %%
+# %% Hypergeometric test
+
+
+# def _prob_hypergeo(datac, yc):
+#     """Compute hypergeometric Pvalue.
+
+#     Description
+#     -----------
+#     Suppose you have a lot of 100 floppy disks (M), and you know that 20 of them are defective (n).
+#     What is the prbability of drawing zero to 2 floppy disks (N=2), if you select 10 at random (N).
+#     P=hypergeom.sf(2,100,20,10)
+
+#     """
+#     P = np.nan
+#     logP = np.nan
+#     M = len(yc)  # Population size: Total number of samples, eg total number of genes; 10000
+#     n = np.sum(datac)  # Number of successes in population, known in pathway, eg 2000
+#     N = np.sum(yc)  # sample size: Random variate, eg clustersize or groupsize, over expressed genes, eg 300
+#     X = np.sum(np.logical_and(yc, datac.values)) - 1  # Let op, de -1 is belangrijk omdatje P<X wilt weten ipv P<=X. Als je P<=X doet dan kan je vele false positives krijgen als bijvoorbeeld X=1 en n=1 oid
+
+#     # Do the hypergeo-test
+#     if np.any(yc) and (X>0):
+#         P = hypergeom.sf(X, M, n, N)
+#         logP = hypergeom.logsf(X, M, n, N)
+
+#     # Store
+#     out = {}
+#     out['category_label']=datac.name
+#     out['P']=P
+#     out['logP']=logP
+#     out['overlap_X']=X
+#     out['popsize_M']=M
+#     out['nr_succes_pop_n']=n
+#     out['samplesize_N']=N
+#     out['dtype']='categorical'
+#     return(out)
+
 
 # %% Naive Bayesian model
-import bnlearn as bn
 df = bn.import_example('random')
-
 # Structure learning
 model = bn.structure_learning.fit(df, methodtype='naivebayes', root_node="B")
+model = bn.independence_test(model, df, prune=True)
 # Plot
 bn.plot(model)
 
-
 # %% Naive Bayesian model
-import bnlearn as bn
-from pgmpy.factors.discrete import TabularCPD
 
 edges = [('A', 'B'), ('A', 'C'), ('A', 'D')]
 DAG = bn.make_DAG(edges, methodtype='naivebayes')
-bn.plot(DAG);
+bn.plot(DAG)
 
 cpd_A = TabularCPD(variable='A', variable_card=3, values=[[0.3], [0.5], [0.2]])
 print(cpd_A)
@@ -40,13 +301,14 @@ bn.print_CPD(DAG, checkmodel=True)
 
 # %% Adjust some edge properties
 
-import bnlearn as bn
 # Load asia DAG
 df = bn.import_example(data='asia')
 # Structure learning of sampled dataset
 model = bn.structure_learning.fit(df)
 # plot static
 G = bn.plot(model)
+# Compute associations with the chi_square test statistic
+model = bn.independence_test(model, df)
 
 # Set some edge properties
 edge_properties = bn.get_edge_properties(model)
@@ -61,14 +323,13 @@ node_properties['xray']['node_color']='#8A0707'
 node_properties['xray']['node_size']=20
 
 # Plot
-params_static={'edge_alpha':0.6, 'arrowstyle':'->', 'arrowsize':60}
+params_static={'edge_alpha': 0.6, 'arrowstyle': '->', 'arrowsize': 60}
 bn.plot(model, interactive=False, node_properties=node_properties, edge_properties=edge_properties, params_static=params_static)
 # bn.plot(model, interactive=True, node_properties=node_properties, edge_properties=edge_properties, params_static=params_static);
 
 # %% TAN : Tree-augmented Naive Bayes (TAN)
 # https://pgmpy.org/examples/Structure%20Learning%20with%20TAN.html
 # https://pgmpy.org/models/naive.html
-import bnlearn as bn
 
 df = bn.import_example()
 # Structure learning
@@ -78,7 +339,6 @@ bn.plot(model, interactive=True, node_size=10)
 
 
 # %% Coloring networks
-import bnlearn as bn
 
 # Load example dataset
 df = bn.import_example(data='asia')
@@ -94,9 +354,9 @@ node_properties['xray']['node_color']='#8A0707'
 node_properties['xray']['node_size']=50
 bn.plot(model, interactive=False)
 bn.plot(model, interactive=False, node_properties=node_properties)
-bn.plot(model, interactive=False, node_properties=node_properties, params_static = {'font_color':'#8A0707'})
-bn.plot(model, interactive=False, params_static = {'width':15, 'height':8, 'font_size':14, 'font_family':'times new roman', 'alpha':0.8, 'node_shape':'o', 'facecolor':'white', 'font_color':'r'})
-bn.plot(model, interactive=False, node_color='#8A0707', node_size=800, params_static = {'width':15, 'height':8, 'font_size':14, 'font_family':'times new roman', 'alpha':0.8, 'node_shape':'o', 'facecolor':'white', 'font_color':'#000000'})
+bn.plot(model, interactive=False, node_properties=node_properties, params_static={'font_color': '#8A0707'})
+bn.plot(model, interactive=False, params_static={'width': 15, 'height': 8, 'font_size': 14, 'font_family': 'times new roman', 'alpha': 0.8, 'node_shape': 'o', 'facecolor': 'white', 'font_color': 'r'})
+bn.plot(model, interactive=False, node_color='#8A0707', node_size=800, params_static={'width': 15, 'height': 8, 'font_size': 14, 'font_family': 'times new roman', 'alpha': 0.8, 'node_shape': 'o', 'facecolor': 'white', 'font_color': '#000000'})
 
 # Add some parameters for the interactive plot
 node_properties = bn.get_node_properties(model)
@@ -107,11 +367,10 @@ bn.plot(model, interactive=True, node_properties=node_properties)
 bn.plot(model, interactive=True, node_color='#8A0707')
 bn.plot(model, interactive=True, node_size=5)
 bn.plot(model, interactive=True, node_properties=node_properties, node_size=20)
-bn.plot(model, interactive=True, params_interactive = {'height':'800px', 'width':'70%', 'layout':None, 'bgcolor':'#0f0f0f0f'})
+bn.plot(model, interactive=True, params_interactive={'height': '800px', 'width': '70%', 'layout': None, 'bgcolor': '#0f0f0f0f'})
 
 
 # %% Save and load trained models
-import bnlearn as bn
 # Import example
 df = bn.import_example(data='asia')
 # Learn structure
@@ -123,33 +382,23 @@ model = bn.load(filepath='bnlearn_model')
 
 
 # %% CHECK DIFFERENCES PGMPY vs. BNLEARN
-import pandas as pd
-import numpy as np
-import matplotlib.pyplot as plt
-from pgmpy.estimators import TreeSearch
-from pgmpy.models import BayesianModel
-import networkx as nx
-import bnlearn as bnlearn
-from pgmpy.inference import VariableElimination
-from pgmpy.estimators import BDeuScore, K2Score, BicScore
-import bnlearn as bn
 
 
-df=bnlearn.import_example(data='andes')
+df=bn.import_example(data='andes')
 
 # PGMPY
 est = TreeSearch(df)
-dag = est.estimate(estimator_type="tan",class_node='DISPLACEM0')
-bnq = BayesianModel(dag.edges())
-bnq.fit(df, estimator=None) # None means maximum likelihood estimator
+dag = est.estimate(estimator_type="tan", class_node='DISPLACEM0')
+bnq = BayesianNetwork(dag.edges())
+bnq.fit(df, estimator=None)  # None means maximum likelihood estimator
 bn_infer = VariableElimination(bnq)
-q = bn_infer.query(variables=['DISPLACEM0'], evidence={'RApp1':1})
+q = bn_infer.query(variables=['DISPLACEM0'], evidence={'RApp1': 1})
 print(q)
 
 # BNLEARN
-model = bnlearn.structure_learning.fit(df, methodtype='tan', class_node='DISPLACEM0', scoretype='bic')
-model_bn = bnlearn.parameter_learning.fit(model, df, methodtype='ml') # maximum likelihood estimator
-query=bnlearn.inference.fit(model_bn, variables=['DISPLACEM0'], evidence={'RApp1':1})
+model = bn.structure_learning.fit(df, methodtype='tan', class_node='DISPLACEM0', scoretype='bic')
+model_bn = bn.parameter_learning.fit(model, df, methodtype='ml')  # maximum likelihood estimator
+query=bn.inference.fit(model_bn, variables=['DISPLACEM0'], evidence={'RApp1': 1})
 
 # DAG COMPARISON
 assert np.all(model_bn['adjmat']==model['adjmat'])
@@ -172,41 +421,37 @@ for cpd_bnlearn in model_bn['model'].get_cpds():
         if cpd_bnlearn.variable==cpd_pgmpy.variable:
             assert np.all(cpd_bnlearn.values==cpd_pgmpy.values)
             # if not np.all(cpd_bnlearn.values==cpd_pgmpy.values):
-                # print('%s-%s'%(cpd_bnlearn.variable, cpd_pgmpy.variable))
-                # print(cpd_bnlearn)
-                # print(cpd_pgmpy)
-                # nr_diff=nr_diff+1
-                # input('press enter to see the next difference in CPD.')
+            # print('%s-%s'%(cpd_bnlearn.variable, cpd_pgmpy.variable))
+            # print(cpd_bnlearn)
+            # print(cpd_pgmpy)
+            # nr_diff=nr_diff+1
+            # input('press enter to see the next difference in CPD.')
 
 
-#%% Example of interactive plotting
-from pgmpy.estimators import TreeSearch
-from pgmpy.models import BayesianModel
-import bnlearn as bn
+# %% Example of interactive plotting
 
 # Import example
 df = bn.import_example(data='asia')
 
 # Do the tan learning
 est = TreeSearch(df)
-dag = est.estimate(estimator_type="tan",class_node='lung')
+dag = est.estimate(estimator_type="tan", class_node='lung')
 
 # And now with bnlearn
 model = bn.structure_learning.fit(df, methodtype='tan', class_node='lung')
 
 # Compare results
 assert dag.edges()==model['model'].edges()
-assert dag.edges()==model['model_edges']
+assert list(dag.edges())==model['model_edges']
 
 # from pgmpy.inference import VariableElimination
-# bnp = BayesianModel(dag.edges())
+# bnp = BayesianNetwork(dag.edges())
 # bn_infer = VariableElimination(bnp)
 # bnp.fit(df)
 # a = bn_infer.query(variables=['Target'], evidence={'X':'c'})
 # print(a)
 
-#%% Example of interactive plotting
-import bnlearn as bn
+# %% Example of interactive plotting
 
 # Load example dataset
 df = bn.import_example(data='asia')
@@ -214,17 +459,18 @@ df = bn.import_example(data='asia')
 # Structure learning
 model = bn.structure_learning.fit(df)
 
+model = bn.independence_test(model, df)
+
 bn.plot(model, interactive=False, node_size=800)
 
 # Add some parameters for the interactive plot
-bn.plot(model, interactive=True, node_size=10, params_interactive = {'height':'600px'})
+bn.plot(model, interactive=True, node_size=10, params_interactive={'height': '600px'})
 
 # Add more parameters for the interactive plot
-bn.plot(model, interactive=True, node_size=10, params_interactive = {'height':'800px', 'width':'70%', 'notebook':False, 'heading':'bnlearn causal diagram', 'layout':None, 'font_color': False, 'bgcolor':'#ffffff'})
+bn.plot(model, interactive=True, node_size=10, params_interactive={'height': '800px', 'width': '70%', 'notebook': False, 'heading': 'bnlearn causal diagram', 'layout': None, 'font_color': False, 'bgcolor': '#ffffff'})
 
 # %% TAN : Tree-augmented Naive Bayes (TAN)
 # https://pgmpy.org/examples/Structure%20Learning%20with%20TAN.html
-import bnlearn as bn
 
 df = bn.import_example()
 # Structure learning
@@ -235,22 +481,20 @@ bn.plot(model, interactive=True)
 
 
 # %% Large dataset
-import pandas as pd
 df=pd.read_csv('c:/temp/features.csv')
-model = bn.structure_learning.fit(df.iloc[:,0:1000], methodtype='cl', root_node='21')
-model = bn.structure_learning.fit(df.iloc[:,0:100], methodtype='cs')
+model = bn.structure_learning.fit(df.iloc[:, 0:1000], methodtype='cl', root_node='21')
+model = bn.structure_learning.fit(df.iloc[:, 0:100], methodtype='cs')
 bn.plot(model)
 
 # %% White_list edges
-import bnlearn as bn
 DAG = bn.import_DAG('asia')
 # plot ground truth
 df = bn.sampling(DAG, n=1000)
 
 # Structure learning with black list
-model = bn.structure_learning.fit(df, methodtype='hc', white_list=[('tub','lung'), ('smoke','bronc')], bw_list_method='edges')
+model = bn.structure_learning.fit(df, methodtype='hc', white_list=[('tub', 'lung'), ('smoke', 'bronc')], bw_list_method='edges')
 bn.plot(model)
-model = bn.structure_learning.fit(df, methodtype='hc', white_list=['tub','lung', 'smoke','bronc'], bw_list_method='nodes')
+model = bn.structure_learning.fit(df, methodtype='hc', white_list=['tub', 'lung', 'smoke', 'bronc'], bw_list_method='nodes')
 bn.plot(model)
 model = bn.structure_learning.fit(df, methodtype='hc')
 bn.plot(model, node_color='#000000')
@@ -258,7 +502,6 @@ bn.plot(model, node_color='#000000')
 
 # %% TAN : Tree-augmented Naive Bayes (TAN)
 # https://pgmpy.org/examples/Structure%20Learning%20with%20TAN.html
-import bnlearn as bn
 
 df = bn.import_example()
 # Structure learning
@@ -267,26 +510,23 @@ bn.plot(model)
 bn.plot(model, interactive=True, node_size=10)
 
 # %% Download example
-import bnlearn as bn
 examples = ['titanic', 'sprinkler', 'alarm', 'andes', 'asia', 'sachs', 'water', 'miserables']
 for example in examples:
     df = bn.import_example(data=example)
     # assert ~df.empty
 
 # %%
-import bnlearn as bn
 df = bn.import_example()
 model = bn.structure_learning.fit(df)
 model = bn.structure_learning.fit(df, methodtype='hc')
 
 # %% Predict
-import bnlearn as bn
 
 df = bn.import_example('asia')
 edges = [('smoke', 'lung'),
-          ('smoke', 'bronc'),
-          ('lung', 'xray'),
-          ('bronc', 'xray')]
+         ('smoke', 'bronc'),
+         ('lung', 'xray'),
+         ('bronc', 'xray')]
 
 
 # Make the actual Bayesian DAG
@@ -295,7 +535,7 @@ model = bn.parameter_learning.fit(DAG, df, verbose=3)
 # Generate some data based on DAG
 df = bn.sampling(model, n=1000)
 # Make predictions
-Pout = bn.predict(model, df, variables=['bronc','xray'])
+Pout = bn.predict(model, df, variables=['bronc', 'xray'])
 # query = bnlearn.inference.fit(model, variables=['bronc','xray'], evidence=evidence, to_df=False, verbose=0)
 # print(query)
 
@@ -319,7 +559,6 @@ bn.topological_sort(DAG)
 bn.topological_sort(DAG, '3')
 
 # %%
-import bnlearn as bn
 DAG = bn.import_DAG('sprinkler', verbose=0)
 
 bn.topological_sort(DAG, 'Rain')
@@ -331,7 +570,6 @@ bn.topological_sort(bn.adjmat2vec(DAG['adjmat']), 'Rain')
 
 # %%
 
-import bnlearn as bn
 DAG = bn.import_DAG('sprinkler')
 df = bn.sampling(DAG, n=1000, verbose=0)
 model = bn.structure_learning.fit(df, methodtype='chow-liu', root_node='Wet_Grass')
@@ -346,7 +584,6 @@ model = bn.structure_learning.fit(df)
 G = bn.plot(model)
 
 # %% Load example dataframe from sprinkler
-import bnlearn as bn
 DAG = bn.import_DAG('sprinkler', verbose=0)
 df = bn.sampling(DAG, n=1000, verbose=0)
 
@@ -370,23 +607,20 @@ model_hc_bic = bn.structure_learning.fit(df, methodtype='cl', root_node='Cloudy'
 G = bn.plot(model)
 
 # %% Load example dataframe from sprinkler
-import bnlearn as bn
 DAG = bn.import_DAG('alarm', verbose=0)
 to_vector = bn.adjmat2vec(DAG['adjmat'])
 to_adjmat = bn.vec2adjmat(to_vector['source'], to_vector['target'])
 
 # %% Load example dataframe from sprinkler
-import bnlearn as bn
 df = bn.import_example('sprinkler')
 # Structure learning
 model = bn.structure_learning.fit(df, verbose=0)
 # Plot
 G = bn.plot(model, verbose=0)
 
-model_hc_bic  = bn.structure_learning.fit(df, methodtype='hc', scoretype='bic', verbose=0)
+model_hc_bic = bn.structure_learning.fit(df, methodtype='hc', scoretype='bic', verbose=0)
 
 # %% Try all methods vs score types
-import bnlearn as bn
 df = bn.import_example()
 
 model_hc_bic = bn.structure_learning.fit(df, methodtype='hc', scoretype='bic')
@@ -404,7 +638,6 @@ G = bn.plot(model_hc_bic, verbose=0)
 bn.compare_networks(model_hc_bic, model_cl, pos=G['pos'], verbose=0)
 
 # %% Example with dataset
-import bnlearn as bn
 DAG = bn.import_DAG('sprinkler', verbose=3)
 # Print cpds
 bn.print_CPD(DAG)
@@ -413,7 +646,6 @@ G = bn.plot(DAG, verbose=0)
 df = bn.sampling(DAG, n=100, verbose=3)
 
 # %% Inference using custom DAG
-import bnlearn as bn
 # Load asia DAG
 df = bn.import_example('asia')
 # from tabulate import tabulate
@@ -426,7 +658,7 @@ edges = [('smoke', 'lung'),
          ('bronc', 'xray')]
 
 # edges = [('smoke', 'xray'),
-         # ('bronc', 'lung')]
+# ('bronc', 'lung')]
 
 # Make the actual Bayesian DAG
 DAG = bn.make_DAG(edges, verbose=0)
@@ -450,11 +682,11 @@ bn.print_CPD(DAG)
 df_sampling = bn.sampling(DAG, n=1000)
 
 # Make inference
-q1 = bn.inference.fit(DAG, variables=['lung'], evidence={'smoke':1}, verbose=3)
-q2 = bn.inference.fit(DAG, variables=['bronc'], evidence={'smoke':1}, verbose=0)
-q3 = bn.inference.fit(DAG, variables=['lung'], evidence={'smoke':1, 'bronc':1})
-q4 = bn.inference.fit(DAG, variables=['bronc','lung'], evidence={'smoke':1, 'xray':0})
-q4 = bn.inference.fit(DAG, variables=['bronc','lung'], evidence={'smoke':0, 'xray':0})
+q1 = bn.inference.fit(DAG, variables=['lung'], evidence={'smoke': 1}, verbose=3)
+q2 = bn.inference.fit(DAG, variables=['bronc'], evidence={'smoke': 1}, verbose=0)
+q3 = bn.inference.fit(DAG, variables=['lung'], evidence={'smoke': 1, 'bronc': 1})
+q4 = bn.inference.fit(DAG, variables=['bronc', 'lung'], evidence={'smoke': 1, 'xray': 0})
+q4 = bn.inference.fit(DAG, variables=['bronc', 'lung'], evidence={'smoke': 0, 'xray': 0})
 
 bn.topological_sort(DAG)
 
@@ -477,15 +709,14 @@ bn.query2df(q4)
 
 df = bn.sampling(DAG, n=100)
 out = bn.predict(DAG, df, variables='bronc')
-out = bn.predict(DAG, df, variables=['bronc','xray'])
-out = bn.predict(DAG, df, variables=['bronc','xray','smoke'])
+out = bn.predict(DAG, df, variables=['bronc', 'xray'])
+out = bn.predict(DAG, df, variables=['bronc', 'xray', 'smoke'])
 
 print('done\n\n')
 print(out)
 
 # %% compute causalities
 # Load asia DAG
-import bnlearn as bn
 df = bn.import_example('asia', verbose=0)
 # print(tabulate(df.head(), tablefmt="grid", headers="keys"))
 # print(df)
@@ -507,7 +738,7 @@ bn.print_CPD(DAG)
 bn.compare_networks(DAG, model, verbose=0)
 
 # Make inference
-q4 = bn.inference.fit(DAG, variables=['bronc','lung'], evidence={'smoke':1, 'xray':0}, verbose=3)
+q4 = bn.inference.fit(DAG, variables=['bronc', 'lung'], evidence={'smoke': 1, 'xray': 0}, verbose=3)
 # q4 = bn.inference.fit(DAG, variables=['bronc','lung','xray'], evidence={'smoke':1}, verbose=3)
 # q4 = bn.inference.fit(DAGnew, variables=['bronc','lung'], evidence={'smoke':0, 'xray':0})
 
@@ -515,7 +746,6 @@ q4 = bn.inference.fit(DAG, variables=['bronc','lung'], evidence={'smoke':1, 'xra
 
 # %% Example compare networks
 # Load asia DAG
-import bnlearn as bn
 DAG = bn.import_DAG('asia')
 # plot ground truth
 G = bn.plot(DAG)
@@ -527,16 +757,16 @@ model = bn.structure_learning.fit(df, verbose=0)
 # Structure learning of sampled dataset
 model_sl = bn.structure_learning.fit(df, methodtype='hc', scoretype='bic')
 # Plot based on structure learning of sampled data
-bn.plot(model_sl, pos=G['pos'], interactive=True, params_interactive = {'height':'800px'})
+bn.plot(model_sl, pos=G['pos'], interactive=True, params_interactive={'height': '800px'})
 # Compare networks and make plot
 bn.compare_networks(model, model_sl, pos=G['pos'])
 
 
 # Structure learning with black list
-model_wl = bn.structure_learning.fit(df, methodtype='hc', white_list=['asia','tub','bronc','xray','smoke'], bw_list_method='edges')
+model_wl = bn.structure_learning.fit(df, methodtype='hc', white_list=['asia', 'tub', 'bronc', 'xray', 'smoke'], bw_list_method='edges')
 bn.plot(model_wl, pos=G['pos'])
 
-model_bl = bn.structure_learning.fit(df, methodtype='hc', black_list=['asia','tub'], bw_list_method='edges')
+model_bl = bn.structure_learning.fit(df, methodtype='hc', black_list=['asia', 'tub'], bw_list_method='edges')
 bn.plot(model_bl, pos=G['pos'])
 
 # Compare models
@@ -544,7 +774,6 @@ bn.compare_networks(model_bl, model_wl, pos=G['pos'])
 
 
 # %% PARAMETER LEARNING
-import bnlearn as bn
 df = bn.import_example()
 DAG = bn.import_DAG('sprinkler', CPD=False)
 model_update = bn.parameter_learning.fit(DAG, df)
@@ -559,14 +788,14 @@ DAG = bn.import_DAG('sprinkler')
 # Read raw data and process
 df_raw = bn.import_example(data='sprinkler')
 df = bn.df2onehot(df_raw, verbose=0)[1]
-df.columns=df.columns.str.replace('_1.0','')
+df.columns=df.columns.str.replace('_1.0', '')
 
 # Learn structure
 DAG = bn.structure_learning.fit(df)
 # Learn CPDs
 model = bn.parameter_learning.fit(DAG, df)
-q1 = bn.inference.fit(model, variables=['Wet_Grass'], evidence={'Rain':1, 'Sprinkler':0, 'Cloudy':1})
-q2 = bn.inference.fit(model, variables=['Wet_Grass','Rain'], evidence={'Sprinkler':1})
+q1 = bn.inference.fit(model, variables=['Wet_Grass'], evidence={'Rain': 1, 'Sprinkler': 0, 'Cloudy': 1})
+q2 = bn.inference.fit(model, variables=['Wet_Grass', 'Rain'], evidence={'Sprinkler': 1})
 
 
 q2.values
@@ -577,18 +806,33 @@ q2.no_to_name,
 
 # %% LOAD BIF FILE
 DAG = bn.import_DAG('water', verbose=0)
+# Sampling
 df = bn.sampling(DAG, n=1000)
-model_update = bn.parameter_learning.fit(DAG, df)
-G = bn.plot(model_update)
-G = bn.plot(model_update, interactive=True)
+# Parameter learning
+model = bn.parameter_learning.fit(DAG, df)
+G = bn.plot(model)
+
+# Test for independence
+model1 = bn.independence_test(model, df, prune=False)
+len(model1['model_edges'])
+G = bn.plot(model1)
+
+# Test for independence
+model2 = bn.independence_test(model, df, prune=True)
+len(model2['model_edges'])
+G = bn.plot(model2)
+
+# Plot
+G = bn.plot(model)
+G = bn.plot(model, interactive=True)
 bn.print_CPD(model_update)
 
 
 # %% INFERENCE
 DAG = bn.import_DAG('sprinkler')
 bn.plot(DAG)
-q1 = bn.inference.fit(DAG, variables=['Wet_Grass'], evidence={'Rain':1, 'Sprinkler':0, 'Cloudy':1})
-q2 = bn.inference.fit(DAG, variables=['Wet_Grass','Rain'], evidence={'Sprinkler': 1})
+q1 = bn.inference.fit(DAG, variables=['Wet_Grass'], evidence={'Rain': 1, 'Sprinkler': 0, 'Cloudy': 1})
+q2 = bn.inference.fit(DAG, variables=['Wet_Grass', 'Rain'], evidence={'Sprinkler': 1})
 
 print(q1)
 print(q1.df)
@@ -600,9 +844,9 @@ print(q2.df)
 DAG = bn.import_DAG('asia')
 # DAG = bn.import_DAG('sprinkler')
 bn.plot(DAG)
-q1 = bn.inference.fit(DAG, variables=['lung'], evidence={'bronc':1, 'smoke':1})
-q2 = bn.inference.fit(DAG, variables=['bronc','lung'], evidence={'smoke':1, 'xray':0, 'tub':1})
-q3 = bn.inference.fit(DAG, variables=['lung'], evidence={'bronc':1, 'smoke':1})
+q1 = bn.inference.fit(DAG, variables=['lung'], evidence={'bronc': 1, 'smoke': 1})
+q2 = bn.inference.fit(DAG, variables=['bronc', 'lung'], evidence={'smoke': 1, 'xray': 0, 'tub': 1})
+q3 = bn.inference.fit(DAG, variables=['lung'], evidence={'bronc': 1, 'smoke': 1})
 
 print(q1)
 print(q1.df)
@@ -611,23 +855,22 @@ print(q2.df)
 
 
 # %% Example with mixed dataset: titanic case
-import bnlearn as bn
 # Load example mixed dataset
 df_raw = bn.import_example(data='titanic')
 # Convert to onehot
 dfhot, dfnum = bn.df2onehot(df_raw)
 
-dfnum.loc[0:50,'Survived'] = 2
+dfnum.loc[0:50, 'Survived'] = 2
 # Structure learning
 # DAG = bn.structure_learning.fit(dfnum, methodtype='cl', black_list=['Embarked','Parch','Name'], root_node='Survived', bw_list_method='nodes')
-DAG = bn.structure_learning.fit(dfnum, methodtype='hc', black_list=['Embarked','Parch','Name'], bw_list_method='edges')
+DAG = bn.structure_learning.fit(dfnum, methodtype='hc', black_list=['Embarked', 'Parch', 'Name'], bw_list_method='edges')
 # Plot
 G = bn.plot(DAG)
 # Parameter learning
 model = bn.parameter_learning.fit(DAG, dfnum)
 # Make inference
-q1 = bn.inference.fit(model, variables=['Survived'], evidence={'Sex':True, 'Pclass':True}, verbose=0)
-q2 = bn.inference.fit(model, variables=['Survived'], evidence={'Sex':0}, verbose=0)
+q1 = bn.inference.fit(model, variables=['Survived'], evidence={'Sex': True, 'Pclass': True}, verbose=0)
+q2 = bn.inference.fit(model, variables=['Survived'], evidence={'Sex': 0}, verbose=0)
 
 print(q1)
 print(q1.df)
@@ -640,7 +883,6 @@ Pout = bn.predict(model, Xtest, variables=['Survived'])
 
 
 # %%
-import bnlearn as bn
 DAG = bn.import_DAG('sprinkler', CPD=True)
 # DAG = bn.import_DAG('asia')
 bn.plot(DAG)
@@ -651,7 +893,6 @@ vector = bn.adjmat2vec(DAG['adjmat'])
 adjmat = bn.vec2adjmat(vector['source'], vector['target'])
 
 # %%
-from pgmpy.factors.discrete import TabularCPD
 edges = [('A', 'E'),
          ('S', 'E'),
          ('E', 'O'),
@@ -665,15 +906,15 @@ bn.plot(DAG)
 
 cpd_A = TabularCPD(variable='A', variable_card=3, values=[[0.3], [0.5], [0.2]])
 print(cpd_A)
-cpd_S = TabularCPD(variable='S', variable_card=2, values=[[0.6],[ 0.4]])
+cpd_S = TabularCPD(variable='S', variable_card=2, values=[[0.6], [0.4]])
 print(cpd_S)
 cpd_E = TabularCPD(variable='E', variable_card=2,
-                           values=[
-                               [0.75,0.72,0.88,0.64,0.70,0.90],
-                               [0.25,0.28,0.12,0.36,0.30,0.10]
-                               ],
-                           evidence=['A', 'S'],
-                           evidence_card=[3, 2])
+                   values=[
+                       [0.75, 0.72, 0.88, 0.64, 0.70, 0.90],
+                       [0.25, 0.28, 0.12, 0.36, 0.30, 0.10]
+                   ],
+                   evidence=['A', 'S'],
+                   evidence_card=[3, 2])
 print(cpd_E)
 
 
@@ -682,13 +923,11 @@ bn.print_CPD(DAG, checkmodel=False)
 
 # %% Create a simple DAG:
 # Building a causal DAG
-import bnlearn as bn
-from pgmpy.factors.discrete import TabularCPD
 
 edges = [('Cloudy', 'Sprinkler'),
-       ('Cloudy', 'Rain'),
-       ('Sprinkler', 'Wet_Grass'),
-       ('Rain', 'Wet_Grass')]
+         ('Cloudy', 'Rain'),
+         ('Sprinkler', 'Wet_Grass'),
+         ('Rain', 'Wet_Grass')]
 
 # DAG = bn.import_DAG('sprinkler')
 DAG = bn.make_DAG(edges)
@@ -714,7 +953,7 @@ print(cpt_rain)
 # Wet Grass
 cpt_wet_grass = TabularCPD(variable='Wet_Grass', variable_card=2,
                            values=[[1, 0.1, 0.1, 0.01],
-                                  [0, 0.9, 0.9, 0.99]],
+                                   [0, 0.9, 0.9, 0.99]],
                            evidence=['Sprinkler', 'Rain'],
                            evidence_card=[2, 2])
 print(cpt_wet_grass)
@@ -725,12 +964,11 @@ DAG = bn.make_DAG(DAG, CPD=[cpt_cloudy, cpt_sprinkler], checkmodel=False)
 DAG = bn.make_DAG(DAG, CPD=[cpt_cloudy, cpt_sprinkler, cpt_rain, cpt_wet_grass])
 bn.print_CPD(DAG)
 
-q1 = bn.inference.fit(DAG, variables=['Wet_Grass'], evidence={'Rain':1, 'Sprinkler':0, 'Cloudy':1})
+q1 = bn.inference.fit(DAG, variables=['Wet_Grass'], evidence={'Rain': 1, 'Sprinkler': 0, 'Cloudy': 1})
 
 # %% Example from sphinx
 # Import dataset
 # Import the library
-import bnlearn as bn
 
 # Define the network structure
 edges = [('Cloudy', 'Sprinkler'),
@@ -747,7 +985,6 @@ bn.plot(DAG)
 bn.print_CPD(DAG)
 
 # Import the library
-from pgmpy.factors.discrete import TabularCPD
 
 # Cloudy
 cpt_cloudy = TabularCPD(variable='Cloudy', variable_card=2, values=[[0.5], [0.5]])
@@ -781,12 +1018,11 @@ DAG = bn.make_DAG(DAG, CPD=[cpt_cloudy, cpt_sprinkler, cpt_rain, cpt_wet_grass])
 bn.print_CPD(DAG)
 
 
-q1 = bn.inference.fit(DAG, variables=['Wet_Grass'], evidence={'Rain':1, 'Sprinkler':0, 'Cloudy':1})
+q1 = bn.inference.fit(DAG, variables=['Wet_Grass'], evidence={'Rain': 1, 'Sprinkler': 0, 'Cloudy': 1})
 
 
 # %% Example to create a Bayesian Network, learn its parameters from data and perform the inference.
 
-import bnlearn as bn
 
 # Import example dataset
 df = bn.import_example('sprinkler')
@@ -817,10 +1053,10 @@ bn.print_CPD(DAGmle)
 bn.print_CPD(DAGbay)
 
 # Make inference
-q1 = bn.inference.fit(DAGmle, variables=['Wet_Grass','Cloudy','Sprinkler'], evidence={'Rain':1})
-q1 = bn.inference.fit(DAGmle, variables=['Cloudy','Wet_Grass','Sprinkler'], evidence={'Rain':1})
-q1 = bn.inference.fit(DAGbay, variables=['Wet_Grass'], evidence={'Rain':1, 'Sprinkler':0, 'Cloudy':1})
+q1 = bn.inference.fit(DAGmle, variables=['Wet_Grass', 'Cloudy', 'Sprinkler'], evidence={'Rain': 1})
+q1 = bn.inference.fit(DAGmle, variables=['Cloudy', 'Wet_Grass', 'Sprinkler'], evidence={'Rain': 1})
+q1 = bn.inference.fit(DAGbay, variables=['Wet_Grass'], evidence={'Rain': 1, 'Sprinkler': 0, 'Cloudy': 1})
 
 print(q1.values)
 
-# %%
\ No newline at end of file
+# %%
